﻿#include "syscommugroupthread.h"
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>

SysCommuGroupThread::SysCommuGroupThread(QObject *iParent) :
    TopClassThreadAbs(iParent)
{

}

SysCommuGroupThread::~SysCommuGroupThread()
{

}

void SysCommuGroupThread::run()
{
    if (invokeName() == "LOAD_DATA") {
        loadData(invokeParameter().toInt());
    } else if (invokeName() == "SAVE_DATA") {
        saveData(invokeParameter().toMap());
    }
}

void SysCommuGroupThread::loadData(int iIdInt)
{
    TDataResponse dataRes;

    TSqlQueryV2 sqlQuery(T_SQLCNT_POOL->getSqlDatabase());
    sqlQuery.begin();
    try {
        TSqlSelectorV2 sqlSelector;
        sqlSelector.setTable("sys_commu_grp");
        sqlSelector.setField("*");
        sqlSelector.setFieldFormat(QVariantMap{{"attr_data", "json"}});
        sqlSelector.setWhere(QString(" id = %1 ").arg(iIdInt));
        QVariantMap dataMap = sqlQuery.selectMap(sqlSelector);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        sqlSelector.clear();
        sqlSelector.setTable("sys_commu_grp_detail");
        sqlSelector.setField("*");
        sqlSelector.setWhere(QString(" commu_grp_id = %1 ").arg(QString::number(iIdInt)));
        QVariantList linkLst = sqlQuery.selectArrayMap(sqlSelector);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        dataMap.insert("MAILINFO_LIST", linkLst);

        sqlQuery.commit();

        dataRes.setData(dataMap);
        setInvokeResult(dataRes.toVariantMap());
        return;
    } catch (const TError &err){
        dataRes.setError(err);
    } catch(...) {
        dataRes.setErrText(ttr("Unknow Error!"));
    }
    sqlQuery.rollback();
    setInvokeResult(dataRes.toVariantMap());
}

void SysCommuGroupThread::saveData(const QVariantMap &iDataMap)
{
    TDataResponse dataRes;
    TSqlQueryV2 sqlQuery(T_SQLCNT_POOL->getSqlDatabase());
    sqlQuery.begin();
    try {
        TSqlSelectorV2 sqlSelector;
        sqlSelector.setTable("sys_commu_grp");
        sqlSelector.setField("COUNT(1)");
        if (iDataMap.value("sys_name").toString().trimmed().isEmpty()) {
            throw TError(ttr("Name must not be empty!"),"ERROR","ALREADY_EXISTS");
        }
        sqlSelector.addWhere("sys_name", iDataMap.value("sys_name"));
        if (!iDataMap.value("id").toString().trimmed().isEmpty()) {
            sqlSelector.addWhere(QString(" id != %1 ").arg(iDataMap.value("id").toString()));
        }
        int count = sqlQuery.selectCount(sqlSelector);
        if (count > 0) {
            throw TError(ttr("'%1' already exists!").arg(iDataMap.value("sys_name").toString()), "ERROR", "ALREADY_EXISTS");
        }

        QStringList fieldLst = iDataMap.keys();
        fieldLst.removeOne("updatePolicy");
        fieldLst.removeOne("MAILINFO_LIST");
        fieldLst.removeOne("id");

        TSqlInserterV2 sqlInserter;
        sqlInserter.setTable("sys_commu_grp");
        sqlInserter.setData(iDataMap);
        sqlInserter.setField(fieldLst);
        sqlInserter.setUpdatePolicy(iDataMap.value("updatePolicy").toMap());
        sqlInserter.setUniqueField("id");
        sqlInserter.setAutoIncrementField("id");

        QVariant mailGroupId = sqlQuery.replaceRow(sqlInserter);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        TSqlDeleterV2 sqlDeleter;
        sqlDeleter.setTable("sys_commu_grp_detail");
        sqlDeleter.setWhere(QString(" commu_grp_id = %1 ").arg(mailGroupId.toInt()));
        sqlQuery.deleteRow(sqlDeleter);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        TSqlInserterV2 linkInserter;
        linkInserter.setTable("sys_commu_grp_detail");
        linkInserter.fieldRef() << "commu_grp_id" << "name" << "address" << "mode" << "src_type" << "src_uid";
        for (QVariant link: iDataMap.value("MAILINFO_LIST").toList()) {
            QVariantMap linkMap = link.toMap();
            QVariantMap insertMap;
            insertMap.insert("commu_grp_id", mailGroupId.toString());
            insertMap.insert("name", linkMap.value("name").toString());
            insertMap.insert("address", linkMap.value("address"));
            insertMap.insert("mode", linkMap.value("mode"));
            insertMap.insert("src_type", linkMap.value("src_type"));
            insertMap.insert("src_uid", linkMap.value("src_uid"));
            //插入to_cc和src的值
            linkInserter.setData(insertMap);
            sqlQuery.insertRow(linkInserter);
            if (sqlQuery.lastError().isValid()) {
                throw sqlQuery.lastError();
            }
        }

        sqlQuery.commit();

        dataRes.setData(mailGroupId);
        setInvokeResult(dataRes.toVariantMap());
        return;
    } catch (const TError &err) {
        dataRes.setError(err);
    } catch (...) {
        dataRes.setErrText(ttr("Unknow Error!"));
    }
    sqlQuery.rollback();
    setInvokeResult(dataRes.toVariantMap());
}
